float4x4 World = float4x4(1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1);
float4x4 View = float4x4(1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1);
float4x4 Projection = float4x4(1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1);

texture tileTexture;
sampler tileSampler = sampler_state {
	texture = <tileTexture>;
	MinFilter = LINEAR;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
	AddressV  = WRAP;
	AddressW  = WRAP;
};
float2 tileResolution = float2(1, 1);

texture mapTexture;
sampler mapSampler = sampler_state {
	texture = <mapTexture>;
	MinFilter = POINT;
	MagFilter = POINT;
	MipFilter = POINT;
	AddressU  = WRAP;
	AddressV  = WRAP;
	AddressW  = WRAP;
};

struct VertexShaderInput {
    float4 Position : POSITION0;
};

struct VertexShaderOutput {
    float4 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input) {
    VertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
	output.Position = mul(viewPosition, Projection);
	output.TexCoord = output.Position.xy / output.Position.w;

    return output;
}

float2 tileCells = float2(4, 4);
float2 mapCells = float2(8, 8);

float4x4 transform = float4x4(3, 0, 0, 0, 0, -3, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1);

float2 frac2(float2 a, float2 b) {
	float2 c = a / b;
	return (c - floor(c)) * b;
}

float4 Texel(float2 texCoord) {
	float2 location = mul(float4(texCoord / tileCells, 0, 1), transform);
	float2 tile = tex2Dlod(mapSampler, float4(location * (tileCells / mapCells), 0, 0)).xy * 255; // ^CID //
	float2 tileOffset = tile / tileCells; // ^CUV //
	float2 scale = 1 / tileCells; // ^C̃XP[ //
	float2 uv = frac2(location, scale); // ^CeNX`W //
	float4 color = tex2Dlod(tileSampler, float4(uv + tileOffset, 0, 0)); // F̌ //
	return color;
}

float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0 {
	return Texel(input.TexCoord);
}

technique Technique1 {
    pass Tile {
        VertexShader = compile vs_3_0 VertexShaderFunction();
        PixelShader = compile ps_3_0 PixelShaderFunction();
    }
}
