﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace XELF {

	class QuadFiller : IDisposable {
		readonly GraphicsDevice graphicsDevice;
		readonly VertexDeclaration declaration;
		public QuadFiller(GraphicsDevice graphicsDevice) {
			this.graphicsDevice = graphicsDevice;
			declaration = new VertexDeclaration(graphicsDevice, VertexPositionTexture.VertexElements);
		}
		readonly VertexPositionTexture[] vertices = new VertexPositionTexture[] {
			new VertexPositionTexture(new Vector3(-1, +1, 0), new Vector2(0, 0)),
			new VertexPositionTexture(new Vector3(+1, +1, 0), new Vector2(1, 0)),
			new VertexPositionTexture(new Vector3(-1, -1, 0), new Vector2(0, 1)),
			new VertexPositionTexture(new Vector3(+1, -1, 0), new Vector2(1, 1)),
		};
		public void Dispose() {
			declaration.Dispose();
		}
		public void Fill(Effect effect) {
			Fill(effect, effect.CurrentTechnique);
		}
		public void Fill(Effect effect, EffectTechnique technique) {
			effect.CurrentTechnique = technique;
			effect.Begin(SaveStateMode.SaveState);
			foreach (var pass in technique.Passes) {
				pass.Begin();
				Fill();
				pass.End();
			}
			effect.End();
		}
		public void Fill() {
			graphicsDevice.VertexDeclaration = declaration;
			graphicsDevice.DrawUserPrimitives(PrimitiveType.TriangleStrip, vertices, 0, 2);
		}
		public void Fill(Effect effect, float x0, float y0, float x1, float y1) {
			effect.Begin();
			foreach (var pass in effect.CurrentTechnique.Passes) {
				pass.Begin();
				Fill(x0, y0, x1, y1);
				pass.End();
			}
			effect.End();
		}
		public void Fill(float x0, float y0, float x1, float y1) {
			VertexPositionTexture[] vertices = new VertexPositionTexture[] {
				new VertexPositionTexture(new Vector3(x0, y1, 0), new Vector2(0, 0)),
				new VertexPositionTexture(new Vector3(x1, y1, 0), new Vector2(1, 0)),
				new VertexPositionTexture(new Vector3(x0, y0, 0), new Vector2(0, 1)),
				new VertexPositionTexture(new Vector3(x1, y0, 0), new Vector2(1, 1)),
			};
			graphicsDevice.VertexDeclaration = declaration;
			graphicsDevice.DrawUserPrimitives(PrimitiveType.TriangleStrip, vertices, 0, 2);
		}
	}

}
